@@ -18,9 +18,10 @@
 
 #include "kservicegroup.h"
 #include "kservicegroup_p.h"
-#include "kservicefactory.h"
-#include "kservicegroupfactory.h"
+#include "kservicefactory_p.h"
+#include "kservicegroupfactory_p.h"
 #include "kservice.h"
+#include "ksycoca_p.h"
 #include <ksycoca.h>
 #include <kdesktopfile.h>
 #include <kconfiggroup.h>
@@ -127,12 +128,12 @@ int KServiceGroupPrivate::childCount() const
                 it != m_serviceList.end(); ++it) {
             KSycocaEntry::Ptr p = *it;
             if (p->isType(KST_KService)) {
-                KService::Ptr service = KService::Ptr(p);
+                KService::Ptr service(static_cast<KService*>(p.data()));
                 if (!service->noDisplay()) {
                     m_childCount++;
                 }
             } else if (p->isType(KST_KServiceGroup)) {
-                KServiceGroup::Ptr serviceGroup = KServiceGroup::Ptr(p);
+                KServiceGroup::Ptr serviceGroup(static_cast<KServiceGroup*>(p.data()));
                 m_childCount += serviceGroup->childCount();
             }
         }
@@ -235,13 +236,13 @@ void KServiceGroupPrivate::load(QDataStream &s)
         Q_FOREACH (const QString &path, groupList) {
             if (path.endsWith(QLatin1Char('/'))) {
                 KServiceGroup::Ptr serviceGroup;
-                serviceGroup = KServiceGroupFactory::self()->findGroupByDesktopPath(path, false);
+                serviceGroup = KSycocaPrivate::self()->serviceGroupFactory()->findGroupByDesktopPath(path, false);
                 if (serviceGroup) {
                     m_serviceList.append(KServiceGroup::SPtr(serviceGroup));
                 }
             } else {
                 KService::Ptr service;
-                service = KServiceFactory::self()->findServiceByDesktopPath(path);
+                service = KSycocaPrivate::self()->serviceFactory()->findServiceByDesktopPath(path);
                 if (service) {
                     m_serviceList.append(KServiceGroup::SPtr(service));
                 }
@@ -263,10 +264,10 @@ void KServiceGroupPrivate::save(QDataStream &s)
     QStringList groupList;
     Q_FOREACH (KSycocaEntry::Ptr p, m_serviceList) {
         if (p->isType(KST_KService)) {
-            KService::Ptr service = KService::Ptr(p);
+            KService::Ptr service(static_cast<KService*>(p.data()));
             groupList.append(service->entryPath());
         } else if (p->isType(KST_KServiceGroup)) {
-            KServiceGroup::Ptr serviceGroup = KServiceGroup::Ptr(p);
+            KServiceGroup::Ptr serviceGroup(static_cast<KServiceGroup*>(p.data()));
             groupList.append(serviceGroup->relPath());
         } else {
             //fprintf(stderr, "KServiceGroup: Unexpected object in list!\n");
@@ -294,7 +295,8 @@ QList<KServiceGroup::Ptr> KServiceGroup::groupEntries(EntriesOptions options)
     List tmp = d->entries(this, sort, options & ExcludeNoDisplay, options & AllowSeparators, options & SortByGenericName);
     foreach (const SPtr &ptr, tmp) {
         if (ptr->isType(KST_KServiceGroup)) {
-            list.append(Ptr(ptr));
+            KServiceGroup::Ptr serviceGroup(static_cast<KServiceGroup*>(ptr.data()));
+            list.append(serviceGroup);
         } else if (ptr->isType(KST_KServiceSeparator)) {
             list.append(KServiceGroup::Ptr(static_cast<KServiceGroup *>(new KSycocaEntry())));
         } else if (sort && ptr->isType(KST_KService)) {
@@ -313,7 +315,7 @@ KService::List KServiceGroup::serviceEntries(EntriesOptions options)
     bool foundService = false;
     foreach (const SPtr &ptr, tmp) {
         if (ptr->isType(KST_KService)) {
-            list.append(KService::Ptr(ptr));
+            list.append(KService::Ptr(static_cast<KService*>(ptr.data())));
             foundService = true;
         } else if (ptr->isType(KST_KServiceSeparator) && foundService) {
             list.append(KService::Ptr(static_cast<KService *>(new KSycocaEntry())));
@@ -355,13 +357,15 @@ static void addItem(KServiceGroup::List &sorted, const KSycocaEntry::Ptr &p, boo
 KServiceGroup::List
 KServiceGroupPrivate::entries(KServiceGroup *group, bool sort, bool excludeNoDisplay, bool allowSeparators, bool sortByGenericName)
 {
+    KSycoca::self()->ensureCacheValid();
+
     // If the entries haven't been loaded yet, we have to reload ourselves
     // together with the entries. We can't only load the entries afterwards
     // since the offsets could have been changed if the database has changed.
 
     KServiceGroup::Ptr grp;
     if (!m_bDeep) {
-        grp = KServiceGroupFactory::self()->findGroupByDesktopPath(path, true);
+        grp = KSycocaPrivate::self()->serviceGroupFactory()->findGroupByDesktopPath(path, true);
 
         group = grp.data();
         if (0 == group) { // No guarantee that we still exist!
@@ -408,7 +412,7 @@ KServiceGroupPrivate::entries(KServiceGroup *group, bool sort, bool excludeNoDis
         size_t ln = strxfrm(key.data(), nameStr.constData(), key.size());
         if (ln != size_t(-1)) {
             key.resize(ln);
-            if ((int)ln >= key.size()) {
+            if (int(ln) >= key.size()) {
                 // didn't fit?
                 ln = strxfrm(key.data(), nameStr.constData(), key.size());
                 if (ln == size_t(-1)) {
@@ -444,7 +448,7 @@ KServiceGroupPrivate::entries(KServiceGroup *group, bool sort, bool excludeNoDis
             QString groupPath = rp + item.mid(1) + QLatin1Char('/');
             // Remove entry from sorted list of services.
             for (SortedContainer::iterator it2 = glist.begin(); it2 != glist.end(); ++it2) {
-                const KServiceGroup::Ptr group = KServiceGroup::Ptr(it2.value());
+                const KServiceGroup::Ptr group(static_cast<KServiceGroup*>(it2.value().data()));
                 if (group->relPath() == groupPath) {
                     glist.erase(it2);
                     break;
@@ -455,7 +459,7 @@ KServiceGroupPrivate::entries(KServiceGroup *group, bool sort, bool excludeNoDis
             // TODO: Remove item from sortOrder-list if not found
             // TODO: This prevents duplicates
             for (SortedContainer::iterator it2 = slist.begin(); it2 != slist.end(); ++it2) {
-                const KService::Ptr service = KService::Ptr(it2.value());
+                const KService::Ptr service(static_cast<KService*>(it2.value().data()));
                 if (service->menuId() == item) {
                     slist.erase(it2);
                     break;
@@ -498,7 +502,7 @@ KServiceGroupPrivate::entries(KServiceGroup *group, bool sort, bool excludeNoDis
                     parseAttribute(*it3,  showEmptyMenu, showInline, showInlineHeader, showInlineAlias, inlineValue);
                 }
                 for (SortedContainer::Iterator it2 = glist.begin(); it2 != glist.end(); ++it2) {
-                    KServiceGroup::Ptr group = KServiceGroup::Ptr(it2.value());
+                    KServiceGroup::Ptr group(static_cast<KServiceGroup*>(it2.value().data()));
                     group->setShowEmptyMenu(showEmptyMenu);
                     group->setAllowInline(showInline);
                     group->setShowInlineHeader(showInlineHeader);
@@ -553,7 +557,7 @@ KServiceGroupPrivate::entries(KServiceGroup *group, bool sort, bool excludeNoDis
                 if (!(*it2)->isType(KST_KServiceGroup)) {
                     continue;
                 }
-                KServiceGroup::Ptr group = KServiceGroup::Ptr(*it2);
+                KServiceGroup::Ptr group(static_cast<KServiceGroup*>((*it2).data()));
                 if (group->relPath() == groupPath) {
                     if (!excludeNoDisplay || !group->noDisplay()) {
                         ++it;
@@ -595,7 +599,7 @@ KServiceGroupPrivate::entries(KServiceGroup *group, bool sort, bool excludeNoDis
                 if (!(*it2)->isType(KST_KService)) {
                     continue;
                 }
-                const KService::Ptr service = KService::Ptr(*it2);
+                const KService::Ptr service(static_cast<KService*>((*it2).data()));
                 if (service->menuId() == item) {
                     if (!excludeNoDisplay || !service->noDisplay()) {
                         addItem(sorted, (*it2), needSeparator);
@@ -657,7 +661,8 @@ QStringList KServiceGroup::layoutInfo() const
 KServiceGroup::Ptr
 KServiceGroup::root()
 {
-    return KServiceGroupFactory::self()->findGroupByDesktopPath(QString::fromLatin1("/"), true);
+    KSycoca::self()->ensureCacheValid();
+    return KSycocaPrivate::self()->serviceGroupFactory()->findGroupByDesktopPath(QString::fromLatin1("/"), true);
 }
 
 KServiceGroup::Ptr
@@ -666,13 +671,15 @@ KServiceGroup::group(const QString &relPath)
     if (relPath.isEmpty()) {
         return root();
     }
-    return KServiceGroupFactory::self()->findGroupByDesktopPath(relPath, true);
+    KSycoca::self()->ensureCacheValid();
+    return KSycocaPrivate::self()->serviceGroupFactory()->findGroupByDesktopPath(relPath, true);
 }
 
 KServiceGroup::Ptr
 KServiceGroup::childGroup(const QString &parent)
 {
-    return KServiceGroupFactory::self()->findGroupByDesktopPath(QString::fromLatin1("#parent#") + parent, true);
+    KSycoca::self()->ensureCacheValid();
+    return KSycocaPrivate::self()->serviceGroupFactory()->findGroupByDesktopPath(QString::fromLatin1("#parent#") + parent, true);
 }
 
 QString KServiceGroup::baseGroupName() const
@@ -697,15 +704,21 @@ public:
     {
     }
 
-    virtual QString name() const
-    {
-        return QLatin1String("separator");
-    }
-
+    virtual QString name() const;
 };
 
+QString KServiceSeparatorPrivate::name() const
+{
+    return QStringLiteral("separator");
+}
+
+
 KServiceSeparator::KServiceSeparator()
-    : KSycocaEntry(*new KServiceSeparatorPrivate(QString::fromLatin1("separator")))
+    : KSycocaEntry(*new KServiceSeparatorPrivate(QStringLiteral("separator")))
+{
+}
+
+KServiceSeparator::~KServiceSeparator()
 {
 }
 
